#!/usr/bin/python3

#
#   Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
#   All rights reserved. Copyright (c) 2016-2020 VECTIONEER.
#
import base64
import hashlib
import json
import tempfile

from motorcortex.reply import Reply
from motorcortex.setup_logger import logger

import os
import queue
from concurrent.futures import ThreadPoolExecutor
from pynng import Req0, TLSConfig
from enum import Enum


class ConnectionState(Enum):
    CONNECTING = 0
    CONNECTION_OK = 1
    CONNECTION_LOST = 2
    CONNECTION_FAILED = 3
    DISCONNECTING = 4
    DISCONNECTED = 5


class Base64Encoder(json.JSONEncoder):
    # pylint: disable=method-hidden
    encoding = 'utf-8'

    def default(self, o):
        if isinstance(o, bytes):
            return base64.b64encode(o).decode(Base64Encoder.encoding)
        return json.JSONEncoder.default(self, o)


class Base64Decoder(json.JSONDecoder):
    # pylint: disable=method-hidden
    encoding = 'utf-8'

    def default(self, o):
        if isinstance(o, bytes):
            return base64.b64decode(o).encode(Base64Decoder.decode())
        return json.Base64Decoder.default(self, o)


class Request(object):

    def __init__(self, protobuf_types, parameter_tree):
        self.__socket = None
        self.__url = None
        self.__connected_lock = None
        self.__protobuf_types = protobuf_types
        self.__parameter_tree = parameter_tree
        self.__pool = ThreadPoolExecutor(max_workers=1)
        self.__clb = ThreadPoolExecutor(max_workers=1)
        self.__connection_state = ConnectionState.DISCONNECTED

    def url(self):
        return self.__url

    def connect(self, url, **kwargs):
        self.__connection_state = ConnectionState.CONNECTING
        conn_timeout_ms, recv_timeout_ms, certificate, state_update = self.parse(**kwargs)
        self.__url = url
        tls_config = None
        if certificate:
            tls_config = TLSConfig(TLSConfig.MODE_CLIENT, ca_files=certificate)

        self.__socket = Req0(recv_timeout=recv_timeout_ms, tls_config=tls_config)
        self.__connected_lock = queue.Queue()

        def pre_connect_cb(pipe):
            self.__connected_lock.put(True)
            self.__connection_state = ConnectionState.CONNECTION_OK
            if state_update:
                self.__clb.submit(state_update, self, self.connectionState())

        def post_remove_cb(pipe):
            if self.__connection_state == ConnectionState.DISCONNECTING:
                self.__connection_state = ConnectionState.DISCONNECTED
            elif self.__connection_state == ConnectionState.CONNECTING:
                self.__connection_state = ConnectionState.CONNECTION_FAILED
            elif self.__connection_state == ConnectionState.CONNECTION_OK:
                self.__connection_state = ConnectionState.CONNECTION_LOST
            self.__connected_lock.put(False)
            if state_update:
                self.__clb.submit(state_update, self, self.connectionState())

        self.__socket.add_post_pipe_connect_cb(pre_connect_cb)
        self.__socket.add_post_pipe_remove_cb(post_remove_cb)
        self.__socket.dial(url)

        return Reply(self.__pool.submit(self.waitForConnection, self.__connected_lock,
                                        conn_timeout_ms / 1000.0))

    def close(self):
        self.__connection_state = ConnectionState.DISCONNECTING
        if self.__connected_lock:
            self.__connected_lock.put(False)
        self.__socket.close()

    def send(self, encoded_msg, do_not_decode_reply=False):
        if self.__socket is not None:
            return Reply(self.__pool.submit(self.__send, self.__socket, encoded_msg,
                                            None if do_not_decode_reply else self.__protobuf_types))
        return None

    def login(self, login, password):
        """Send a login request to the server

            Args:
                login(str): user login
                password(str): user password

            Results:
                Reply(StatusMsg): A Promise, which resolves if login is successful and fails otherwise.
                Returned message has a status code, which indicates a status of the login.

            Examples:
                >>> login_reply = req.login('operator', 'iddqd')
                >>> login_msg = login_reply.get()
                >>> if login_msg.status == motorcortex_msg.OK
                >>>     print('User logged-in')

        """

        login_msg = self.__protobuf_types.createType('motorcortex.LoginMsg')
        login_msg.password = password
        login_msg.login = login

        return self.send(self.__protobuf_types.encode(login_msg))

    def connectionState(self):
        return self.__connection_state

    def getParameterTreeHash(self):
        """Request a parameter tree hash from the server.

            Returns:
                Reply(ParameterTreeMsg): A Promise, which resolves when parameter tree hash is received or fails
                otherwise. ParameterTreeHashMsg message has a status field to check the status of the operation.

            Examples:
                >>> param_tree_hash_reply = req.getParameterTreeHash()
                >>> value = param_tree_hash_reply.get()

        """

        # getting and instantiating data type from the loaded dict
        param_tree_hash_msg = self.__protobuf_types.createType('motorcortex.GetParameterTreeHashMsg')

        # encoding and sending data
        return self.send(self.__protobuf_types.encode(param_tree_hash_msg))

    def getParameterTree(self):
        """Request a parameter tree from the server.

            Returns:
                Reply(ParameterTreeMsg): A Promise, which resolves when parameter tree is received or fails
                otherwise. ParameterTreeMsg message has a status field to check the status of the operation.

            Examples:
                >>> param_tree_reply = req.getParameterTree()
                >>> value = param_tree_reply.get()
                >>> parameter_tree.load(value)

        """

        return Reply(self.__pool.submit(self.__getParameterTree,
                                        self.getParameterTreeHash(), self.__protobuf_types, self.__socket))

    def save(self, path, file_name):
        """Request a server to save a parameter tree to file.

            Args:
                path(str): path where to save
                file_name(str): file name

            Returns:
                Reply(StatusMsg): A promise, which resolves when save operation is completed,
                fails otherwise.

        """

        param_save_msg = self.__protobuf_types.createType('motorcortex.SaveMsg')
        param_save_msg.path = path
        param_save_msg.file_name = file_name

        return self.send(self.__protobuf_types.encode(param_save_msg))

    def setParameter(self, path, value, type_name=None, offset=0, length=0):
        """Set new value to a parameter with given path

            Args:
                path(str): parameter path in the tree
                value(any): new parameter value
                type_name(str): type of the value (by default resolved automatically)
                offset(int): offset of the elements to update in the destination array, (by default is 0)
                length(int): number of the elements to update in the destination array, (by default takes length of the
                value argument)

            Returns:
                  Reply(StatusMsg): A Promise, which resolves when parameter value is updated or fails otherwise.

            Examples:
                  >>> reply = req.setParameter("root/Control/activateSemiAuto", False)
                  >>> reply.get()
                  >>> reply = req.setParameter("root/Control/targetJointAngles", [0.2, 3.14, 0.4])
                  >>> reply.get()
        """

        if (offset == 0) and (length == 0):
            return self.send(self.__protobuf_types.encode(self.__buildSetParameterMsg(path, value,
                                                                                      type_name, self.__protobuf_types,
                                                                                      self.__parameter_tree)))
        else:
            return self.send(
                self.__protobuf_types.encode(self.__buildSetParameterWithOffsetMsg(offset, length, path, value,
                                                                                   type_name, self.__protobuf_types,
                                                                                   self.__parameter_tree)))

    def setParameterList(self, param_list):
        """Set new values to a parameter list

            Args:
                 param_list([{'path'-`str`,'value'-`any`}]): a list of the parameters which values update

            Returns:
                Reply(StatusMsg): A Promise, which resolves when parameters from the list are updated,
                otherwise fails.

            Examples:
                  >>>  req.setParameterList([
                  >>>   {'path': 'root/Control/generator/enable', 'value': False},
                  >>>   {'path': 'root/Control/generator/amplitude', 'value': 1.4}])

        """

        # instantiating message type
        set_param_list_msg = self.__protobuf_types.createType("motorcortex.SetParameterListMsg")
        # filling with sub messages
        for param in param_list:
            type_name = None
            if "type_name" in param:
                type_name = param["type_name"]
            set_param_list_msg.params.extend([self.__buildSetParameterMsg(param["path"], param["value"],
                                                                          type_name, self.__protobuf_types,
                                                                          self.__parameter_tree)])

        # encoding and sending data
        return self.send(self.__protobuf_types.encode(set_param_list_msg))

    def getParameter(self, path):
        """Request a parameter with description and value from the server.

            Args:
                path(str): parameter path in the tree.

            Returns:
                 Reply(ParameterMsg): Returns a Promise, which resolves when parameter
                 message is successfully obtained, fails otherwise.

            Examples:
                >>> param_reply = req.getParameter('root/Control/actualActuatorPositions')
                >>> param_full = param_reply.get() # Value and description
        """

        return self.send(self.__protobuf_types.encode(self.__buildGetParameterMsg(path, self.__protobuf_types)))

    def getParameterList(self, path_list):
        """Get description and values of requested parameters.

            Args:
                path_list(str): list of parameter paths in the tree.

            Returns:
                Reply(ParameterListMsg): A Promise, which resolves when list of the parameter values is received, fails
                otherwise.

            Examples:
                >>> params_reply = req.getParameter(['root/Control/joint1', 'root/Control/joint2'])
                >>> params_full = params_reply.get() # Values and descriptions
                >>> print(params_full.params)
        """

        # instantiating message type
        get_param_list_msg = self.__protobuf_types.createType('motorcortex.GetParameterListMsg')
        # filling with sub messages
        for path in path_list:
            get_param_list_msg.params.extend([self.__buildGetParameterMsg(path, self.__protobuf_types)])

        # encoding and sending data
        return self.send(self.__protobuf_types.encode(get_param_list_msg))

    def overwriteParameter(self, path, value, force_activate=False, type_name=None):
        """Overwrites actual value of the parameter and depending on the flag forces this value to stay active.
           This method of setting values is useful during debug and installation process, it is not recommended to use
           this method during normal operation.

            Args:
                path(str): parameter path in the tree
                value(any): new parameter value
                force_activate(bool): forces new value to stay active. (by default is set to 'False')
                type_name(str): type of the value (by default resolved automatically)

            Returns:
                  Reply(StatusMsg): A Promise, which resolves when parameter value is updated or fails otherwise.

            Examples:
                  >>> reply = req.overwriteParameter("root/Control/dummyBool", False, True)
                  >>> reply.get()
        """

        return self.send(self.__protobuf_types.encode(self.__buildOverwriteParameterMsg(path, value, force_activate,
                                                                                        type_name,
                                                                                        self.__protobuf_types,
                                                                                        self.__parameter_tree)))

    def releaseParameter(self, path):
        """Deactivate overwrite operation of the parameter.

            Args:
                path(str): parameter path in the tree

            Returns:
                  Reply(StatusMsg): A Promise, which resolves when parameter value is released or fails otherwise.

            Examples:
                  >>> reply = req.releaseParameter("root/Control/dummyBool")
                  >>> reply.get()
        """

        return self.send(self.__protobuf_types.encode(self.__buildReleaseParameterMsg(path, self.__protobuf_types)))

    def createGroup(self, path_list, group_alias, frq_divider=1):
        """Create a subscription group for a list of the parameters.

            This method is used inside Subscription class, use subscription class instead.

            Args:
                path_list(list(str)): list of the parameters to subscribe to
                group_alias(str): name of the group
                frq_divider(int): frequency divider is a downscaling factor for the group publish rate

            Returns:
                Reply(GroupStatusMsg): A Promise, which resolves when subscription is complete,
                fails otherwise.
        """

        # instantiating message type
        create_group_msg = self.__protobuf_types.createType('motorcortex.CreateGroupMsg')
        create_group_msg.alias = group_alias
        create_group_msg.paths.extend(path_list if type(path_list) is list else [path_list])
        create_group_msg.frq_divider = frq_divider if frq_divider > 1 else 1
        # encoding and sending data
        return self.send(self.__protobuf_types.encode(create_group_msg))

    def removeGroup(self, group_alias):
        """Unsubscribe from the group.

            This method is used inside Subscription class, use subscription class instead.

            Args:
                group_alias(str): name of the group to unsubscribe from

            Returns:
                Reply(StatusMsg): A Promise, which resolves when the unsubscribe operation is complete,
                fails otherwise.
        """

        # instantiating message type
        remove_group_msg = self.__protobuf_types.createType('motorcortex.RemoveGroupMsg')
        remove_group_msg.alias = group_alias
        # encoding and sending data
        return self.send(self.__protobuf_types.encode(remove_group_msg))

    @staticmethod
    def __buildSetParameterMsg(path, value, type_name, protobuf_types, parameter_tree):
        param_value = None
        if not type_name:
            type_id = parameter_tree.getDataType(path)
            if type_id:
                param_value = protobuf_types.getTypeByHash(type_id)
        else:
            param_value = protobuf_types.createType(type_name)

        if not param_value:
            logger.error("Failed to find encoder for the path: %s type: %s" % (path, type_name))

        # creating type instance
        set_param_msg = protobuf_types.createType("motorcortex.SetParameterMsg")
        set_param_msg.path = path
        # encoding parameter value
        set_param_msg.value = param_value.encode(value)

        return set_param_msg

    @staticmethod
    def __buildSetParameterWithOffsetMsg(offset, length, path, value, type_name, protobuf_types, parameter_tree):
        param_value = None
        if not type_name:
            type_id = parameter_tree.getDataType(path)
            if type_id:
                param_value = protobuf_types.getTypeByHash(type_id)
        else:
            param_value = protobuf_types.createType(type_name)

        if not param_value:
            logger.error("Failed to find encoder for the path: %s type: %s" % (path, type_name))

        # check an offset
        if offset < 0:
            offset = 0

        # check length, if == 0 assign length of the value
        if length < 0:
            length = 0
        if length == 0:
            if hasattr(value, '__len__'):
                length = len(value)
            else:
                length = 1

        # creating type instance
        set_param_msg = protobuf_types.createType("motorcortex.SetParameterMsg")
        set_param_msg.offset.type = 1
        set_param_msg.offset.offset = offset
        set_param_msg.offset.length = length
        set_param_msg.path = path
        # encoding parameter value
        set_param_msg.value = param_value.encode(value)

        return set_param_msg

    @staticmethod
    def __buildGetParameterMsg(path, protobuf_types):
        # getting and instantiating data type from the loaded dict
        get_param_msg = protobuf_types.createType('motorcortex.GetParameterMsg')
        get_param_msg.path = path

        return get_param_msg

    @staticmethod
    def __buildOverwriteParameterMsg(path, value, activate, type_name, protobuf_types, parameter_tree):
        param_value = None
        if not type_name:
            type_id = parameter_tree.getDataType(path)
            if type_id:
                param_value = protobuf_types.getTypeByHash(type_id)
        else:
            param_value = protobuf_types.createType(type_name)

        if not param_value:
            logger.error("Failed to find encoder for the path: %s type: %s" % (path, type_name))

        # creating type instance
        overwrite_param_msg = protobuf_types.createType("motorcortex.OverwriteParameterMsg")
        overwrite_param_msg.path = path
        overwrite_param_msg.activate = activate
        # encoding parameter value
        overwrite_param_msg.value = param_value.encode(value)

        return overwrite_param_msg

    @staticmethod
    def __buildReleaseParameterMsg(path, protobuf_types):
        release_param_msg = protobuf_types.createType('motorcortex.ReleaseParameterMsg')
        release_param_msg.path = path

        return release_param_msg

    @staticmethod
    def parse(conn_timeout_ms=0, timeout_ms=None, recv_timeout_ms=None, certificate=None, login=None, password=None,
              state_update=None):
        if timeout_ms and not conn_timeout_ms:
            conn_timeout_ms = timeout_ms

        return conn_timeout_ms, recv_timeout_ms, certificate, state_update

    @staticmethod
    def __send(req, encoded_msg, protobuf_types):
        req.send(encoded_msg)
        buffer = req.recv()
        if buffer:
            if protobuf_types:
                return protobuf_types.decode(buffer)
            else:
                return protobuf_types

        return None

    @staticmethod
    def waitForConnection(connected_lock, timeout_sec):
        if timeout_sec <= 0:
            timeout_sec = None
        try:
            return connected_lock.get(block=True, timeout=timeout_sec)
        except queue.Empty:
            return False

    @staticmethod
    def __getParameterTree(hash_reply, protobuf_types, socket):
        tree_hash = hash_reply.get()
        path = os.sep.join([tempfile.gettempdir(), "mcx-python-pt-" + str(tree_hash.hash)])
        tree = Request.loadParameterTreeFile(path, protobuf_types)
        if tree:
            logger.debug('Found parameter tree in the cache')
            return tree
        else:
            logger.debug('Failed to find parameter tree in the cache')

        # getting and instantiating data type from the loaded dict
        param_tree_msg = protobuf_types.createType('motorcortex.GetParameterTreeMsg')
        handle = Request.__send(socket, protobuf_types.encode(param_tree_msg), protobuf_types)

        # encoding and sending data
        return Request.saveParameterTreeFile(path, handle)

    @staticmethod
    def saveParameterTreeFile(path, parameter_tree):
        logger.debug('Saved parameter tree to the cache')
        json_data = {}
        base64_data = base64.b64encode(parameter_tree.SerializeToString())
        json_data['md5'] = hashlib.md5(base64_data).hexdigest()
        json_data['data'] = base64_data.decode('utf-8')

        with open(path, "w") as outfile:
            outfile.write(json.dumps(json_data))

        return parameter_tree

    @staticmethod
    def loadParameterTreeFile(path, protobuf_types):
        logger.debug('Loaded parameter tree from the cache')
        param_tree_hash_msg = None
        if os.path.exists(path):
            with open(path, "r") as outfile:
                json_data = json.load(outfile)

            if json_data:
                if "md5" in json_data and "data" in json_data:
                    if hashlib.md5(json_data['data'].encode()).hexdigest() == json_data['md5']:
                        param_tree_hash_msg = protobuf_types.createType('motorcortex.ParameterTreeMsg')
                        tree_raw = base64.b64decode(json_data['data'])
                        param_tree_hash_msg.ParseFromString(tree_raw)

        return param_tree_hash_msg



#!/usr/bin/python3

import motorcortex

if __name__ == '__main__':
    parameter_tree = motorcortex.ParameterTree()
    motorcortex_types = motorcortex.MessageTypes()

    # Vectioneer camera
    endpoint_1 = "ws://motorcortex:5558:5557"
    req, sub = motorcortex.connect(endpoint_1, motorcortex_types, parameter_tree,
                                   certificate="mcx.cert.pem", timeout_ms=1000,
                                   login="", password="")


    # Local processing
    endpoint_2 = "wss://localhost:5568:5567"
    req1, sub1 = motorcortex.connect(endpoint_2, motorcortex.MessageTypes(), motorcortex.ParameterTree(),
                                   certificate="mcx.cert.pem", timeout_ms=1000,
                                   login="", password="")

    print("Sending data from {} to {}".format(endpoint_1, endpoint_2))

    counter = 0
    while True:
        ss = req.getParameter("root/Camera/image")
        res = ss.get()
        print("{}: {}".format(counter, len(res.value)))
        handle = req1.setParameter("root/Processing/image", res.value)
        res = handle.get()
        counter = counter + 1




# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: motionSL.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='motionSL.proto',
  package='motion_spec',
  syntax='proto2',
  serialized_pb=_b('\n\x0emotionSL.proto\x12\x0bmotion_spec\"h\n\x19VelAndAccConstraintValues\x12\x0c\n\x04vMax\x18\x01 \x02(\x01\x12\x0c\n\x04\x61Max\x18\x02 \x02(\x01\x12\x10\n\x08omegaMax\x18\x03 \x01(\x01\x12\x0f\n\x07\x61lfaMax\x18\x04 \x01(\x01\x12\x0c\n\x04jMax\x18\x05 \x01(\x01\">\n\x1aTimeAndAccConstraintValues\x12\x12\n\nT_duration\x18\x01 \x02(\x01\x12\x0c\n\x04\x61Max\x18\x02 \x02(\x01\"C\n\x1eTimeAndTimeAccConstraintValues\x12\x12\n\nT_duration\x18\x01 \x02(\x01\x12\r\n\x05T_acc\x18\x02 \x02(\x01\"\x85\x02\n\x10MotionConstraint\x12/\n\x04type\x18\x01 \x02(\x0e\x32!.motion_spec.MOTIONCONSTRAINTTYPE\x12@\n\x0btime_values\x18\x02 \x01(\x0b\x32+.motion_spec.TimeAndTimeAccConstraintValues\x12?\n\x0etimeacc_values\x18\x03 \x01(\x0b\x32\'.motion_spec.TimeAndAccConstraintValues\x12=\n\rvelacc_values\x18\x04 \x01(\x0b\x32&.motion_spec.VelAndAccConstraintValues\"W\n\x12WayPointConstraint\x12\x31\n\x04type\x18\x01 \x02(\x0e\x32#.motion_spec.WAYPOINTCONSTRAINTTYPE\x12\x0e\n\x06\x66\x61\x63tor\x18\x02 \x02(\x01\"?\n\tCartPose6\x12\x13\n\x0b\x63oordinates\x18\x01 \x03(\x01\x12\n\n\x02id\x18\x02 \x01(\r\x12\x11\n\tid_parent\x18\x03 \x01(\r\"\x1c\n\x05Twist\x12\x13\n\x0b\x63oordinates\x18\x01 \x03(\x01\"D\n\x0eJointPositions\x12\x13\n\x0b\x63oordinates\x18\x01 \x03(\x01\x12\n\n\x02id\x18\x02 \x01(\r\x12\x11\n\tid_parent\x18\x03 \x01(\r\"&\n\x0fJointVelocities\x12\x13\n\x0b\x63oordinates\x18\x01 \x03(\x01\"u\n\rToolTipOffset\x12\x0f\n\x07offsetX\x18\x01 \x02(\x01\x12\x0f\n\x07offsetY\x18\x02 \x02(\x01\x12\x0f\n\x07offsetZ\x18\x03 \x02(\x01\x12\x0f\n\x07offsetA\x18\x04 \x02(\x01\x12\x0f\n\x07offsetB\x18\x05 \x02(\x01\x12\x0f\n\x07offsetC\x18\x06 \x02(\x01\"\x82\x01\n\x0c\x43\x61rtWaypoint\x12$\n\x04pose\x18\x01 \x02(\x0b\x32\x16.motion_spec.CartPose6\x12\x33\n\nconstraint\x18\x02 \x01(\x0b\x32\x1f.motion_spec.WayPointConstraint\x12\x17\n\x0fsegmentVelocity\x18\x03 \x01(\x01\"\x88\x01\n\rJointWaypoint\x12)\n\x04pose\x18\x01 \x02(\x0b\x32\x1b.motion_spec.JointPositions\x12\x33\n\nconstraint\x18\x02 \x01(\x0b\x32\x1f.motion_spec.WayPointConstraint\x12\x17\n\x0fsegmentVelocity\x18\x03 \x01(\x01\"\xdb\x01\n\x05MoveC\x12\x31\n\nconstraint\x18\x01 \x02(\x0b\x32\x1d.motion_spec.MotionConstraint\x12\x33\n\x0ereferenceJoint\x18\x02 \x02(\x0b\x32\x1b.motion_spec.JointPositions\x12,\n\twaypoints\x18\x03 \x03(\x0b\x32\x19.motion_spec.CartWaypoint\x12\r\n\x05\x61ngle\x18\x04 \x02(\x01\x12-\n\tuserframe\x18\x05 \x01(\x0e\x32\x1a.motion_spec.USERFRAMETYPE\"\xcc\x01\n\x05MoveL\x12\x31\n\nconstraint\x18\x01 \x02(\x0b\x32\x1d.motion_spec.MotionConstraint\x12\x33\n\x0ereferenceJoint\x18\x02 \x02(\x0b\x32\x1b.motion_spec.JointPositions\x12,\n\twaypoints\x18\x03 \x03(\x0b\x32\x19.motion_spec.CartWaypoint\x12-\n\tuserframe\x18\x05 \x01(\x0e\x32\x1a.motion_spec.USERFRAMETYPE\"i\n\x05MoveJ\x12\x31\n\nconstraint\x18\x01 \x02(\x0b\x32\x1d.motion_spec.MotionConstraint\x12-\n\twaypoints\x18\x02 \x03(\x0b\x32\x1a.motion_spec.JointWaypoint\"4\n\x03Set\x12\x0c\n\x04path\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x02(\x08\x12\x10\n\x08time_sec\x18\x03 \x01(\x01\"\x13\n\x03Get\x12\x0c\n\x04path\x18\x01 \x02(\t\"H\n\x04Wait\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08\x12\x0f\n\x07timeout\x18\x03 \x01(\x01\x12\x12\n\nerror_code\x18\x04 \x01(\r\"c\n\rMotionCommand\x12\n\n\x02id\x18\x01 \x02(\r\x12,\n\x0b\x63ommandtype\x18\x02 \x02(\x0e\x32\x17.motion_spec.MOTIONTYPE\x12\x18\n\x10\x63ommandarguments\x18\x03 \x02(\x0c\"Z\n\rMotionProgram\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\n\n\x02id\x18\x02 \x02(\r\x12/\n\x0b\x63ommandlist\x18\x03 \x03(\x0b\x32\x1a.motion_spec.MotionCommand\"\xf3\x01\n\x0b\x43\x61rtToJoint\x12(\n\x08\x63\x61rtpose\x18\x01 \x02(\x0b\x32\x16.motion_spec.CartPose6\x12.\n\tjointpose\x18\x02 \x02(\x0b\x32\x1b.motion_spec.JointPositions\x12\x31\n\nframe_type\x18\x03 \x02(\x0e\x32\x1d.motion_spec.FEATUREFRAMETYPE\x12\x30\n\njointtwist\x18\x04 \x02(\x0b\x32\x1c.motion_spec.JointVelocities\x12%\n\tcarttwist\x18\x05 \x02(\x0b\x32\x12.motion_spec.Twist\"D\n\x0f\x43\x61rtToJointList\x12\x31\n\x0f\x63\x61rttojointlist\x18\x01 \x03(\x0b\x32\x18.motion_spec.CartToJoint\"\xc0\x01\n\x0bJointToCart\x12(\n\x08\x63\x61rtpose\x18\x01 \x02(\x0b\x32\x16.motion_spec.CartPose6\x12.\n\tjointpose\x18\x02 \x02(\x0b\x32\x1b.motion_spec.JointPositions\x12\x30\n\njointtwist\x18\x03 \x02(\x0b\x32\x1c.motion_spec.JointVelocities\x12%\n\tcarttwist\x18\x04 \x02(\x0b\x32\x12.motion_spec.Twist\"D\n\x0fJointToCartList\x12\x31\n\x0fjointtocartlist\x18\x01 \x03(\x0b\x32\x18.motion_spec.JointToCart*I\n\nMOTIONTYPE\x12\x0f\n\x0bJOINTMOTION\x10\x00\x12\x0e\n\nCARTMOTION\x10\x01\x12\x07\n\x03SET\x10\x02\x12\x08\n\x04WAIT\x10\x03\x12\x07\n\x03\x41RC\x10\x04*\x80\x01\n\rUSERFRAMETYPE\x12\t\n\x05ROBOT\x10\x00\x12\x07\n\x03UF1\x10\x01\x12\x07\n\x03UF2\x10\x02\x12\x07\n\x03UF3\x10\x03\x12\x07\n\x03UF4\x10\x04\x12\x07\n\x03UF5\x10\x05\x12\x07\n\x03UF6\x10\x06\x12\x07\n\x03UF7\x10\x07\x12\x07\n\x03UF8\x10\x08\x12\x07\n\x03UF9\x10\t\x12\x08\n\x04UF10\x10\n\x12\t\n\x05WORLD\x10\x64*B\n\x16WAYPOINTCONSTRAINTTYPE\x12\x0c\n\x08POSITION\x10\x00\x12\x08\n\x04TIME\x10\x01\x12\x10\n\x0c\x41\x43\x43\x45LERATION\x10\x02*I\n\x14MOTIONCONSTRAINTTYPE\x12\x0e\n\nTIMEANDACC\x10\x00\x12\x12\n\x0eTIMEANDTIMEACC\x10\x01\x12\r\n\tVELANDACC\x10\x02*+\n\x10\x46\x45\x41TUREFRAMETYPE\x12\n\n\x06\x46LANGE\x10\x00\x12\x0b\n\x07TOOLTIP\x10\x01')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

_MOTIONTYPE = _descriptor.EnumDescriptor(
  name='MOTIONTYPE',
  full_name='motion_spec.MOTIONTYPE',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='JOINTMOTION', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CARTMOTION', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SET', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WAIT', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ARC', index=4, number=4,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=2678,
  serialized_end=2751,
)
_sym_db.RegisterEnumDescriptor(_MOTIONTYPE)

MOTIONTYPE = enum_type_wrapper.EnumTypeWrapper(_MOTIONTYPE)
_USERFRAMETYPE = _descriptor.EnumDescriptor(
  name='USERFRAMETYPE',
  full_name='motion_spec.USERFRAMETYPE',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ROBOT', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UF1', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UF2', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UF3', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UF4', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UF5', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UF6', index=6, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UF7', index=7, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UF8', index=8, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UF9', index=9, number=9,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UF10', index=10, number=10,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WORLD', index=11, number=100,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=2754,
  serialized_end=2882,
)
_sym_db.RegisterEnumDescriptor(_USERFRAMETYPE)

USERFRAMETYPE = enum_type_wrapper.EnumTypeWrapper(_USERFRAMETYPE)
_WAYPOINTCONSTRAINTTYPE = _descriptor.EnumDescriptor(
  name='WAYPOINTCONSTRAINTTYPE',
  full_name='motion_spec.WAYPOINTCONSTRAINTTYPE',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='POSITION', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TIME', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ACCELERATION', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=2884,
  serialized_end=2950,
)
_sym_db.RegisterEnumDescriptor(_WAYPOINTCONSTRAINTTYPE)

WAYPOINTCONSTRAINTTYPE = enum_type_wrapper.EnumTypeWrapper(_WAYPOINTCONSTRAINTTYPE)
_MOTIONCONSTRAINTTYPE = _descriptor.EnumDescriptor(
  name='MOTIONCONSTRAINTTYPE',
  full_name='motion_spec.MOTIONCONSTRAINTTYPE',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TIMEANDACC', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TIMEANDTIMEACC', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VELANDACC', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=2952,
  serialized_end=3025,
)
_sym_db.RegisterEnumDescriptor(_MOTIONCONSTRAINTTYPE)

MOTIONCONSTRAINTTYPE = enum_type_wrapper.EnumTypeWrapper(_MOTIONCONSTRAINTTYPE)
_FEATUREFRAMETYPE = _descriptor.EnumDescriptor(
  name='FEATUREFRAMETYPE',
  full_name='motion_spec.FEATUREFRAMETYPE',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='FLANGE', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TOOLTIP', index=1, number=1,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=3027,
  serialized_end=3070,
)
_sym_db.RegisterEnumDescriptor(_FEATUREFRAMETYPE)

FEATUREFRAMETYPE = enum_type_wrapper.EnumTypeWrapper(_FEATUREFRAMETYPE)
JOINTMOTION = 0
CARTMOTION = 1
SET = 2
WAIT = 3
ARC = 4
ROBOT = 0
UF1 = 1
UF2 = 2
UF3 = 3
UF4 = 4
UF5 = 5
UF6 = 6
UF7 = 7
UF8 = 8
UF9 = 9
UF10 = 10
WORLD = 100
POSITION = 0
TIME = 1
ACCELERATION = 2
TIMEANDACC = 0
TIMEANDTIMEACC = 1
VELANDACC = 2
FLANGE = 0
TOOLTIP = 1



_VELANDACCCONSTRAINTVALUES = _descriptor.Descriptor(
  name='VelAndAccConstraintValues',
  full_name='motion_spec.VelAndAccConstraintValues',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='vMax', full_name='motion_spec.VelAndAccConstraintValues.vMax', index=0,
      number=1, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='aMax', full_name='motion_spec.VelAndAccConstraintValues.aMax', index=1,
      number=2, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='omegaMax', full_name='motion_spec.VelAndAccConstraintValues.omegaMax', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='alfaMax', full_name='motion_spec.VelAndAccConstraintValues.alfaMax', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='jMax', full_name='motion_spec.VelAndAccConstraintValues.jMax', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=31,
  serialized_end=135,
)


_TIMEANDACCCONSTRAINTVALUES = _descriptor.Descriptor(
  name='TimeAndAccConstraintValues',
  full_name='motion_spec.TimeAndAccConstraintValues',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='T_duration', full_name='motion_spec.TimeAndAccConstraintValues.T_duration', index=0,
      number=1, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='aMax', full_name='motion_spec.TimeAndAccConstraintValues.aMax', index=1,
      number=2, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=137,
  serialized_end=199,
)


_TIMEANDTIMEACCCONSTRAINTVALUES = _descriptor.Descriptor(
  name='TimeAndTimeAccConstraintValues',
  full_name='motion_spec.TimeAndTimeAccConstraintValues',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='T_duration', full_name='motion_spec.TimeAndTimeAccConstraintValues.T_duration', index=0,
      number=1, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='T_acc', full_name='motion_spec.TimeAndTimeAccConstraintValues.T_acc', index=1,
      number=2, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=201,
  serialized_end=268,
)


_MOTIONCONSTRAINT = _descriptor.Descriptor(
  name='MotionConstraint',
  full_name='motion_spec.MotionConstraint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='motion_spec.MotionConstraint.type', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='time_values', full_name='motion_spec.MotionConstraint.time_values', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timeacc_values', full_name='motion_spec.MotionConstraint.timeacc_values', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='velacc_values', full_name='motion_spec.MotionConstraint.velacc_values', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=271,
  serialized_end=532,
)


_WAYPOINTCONSTRAINT = _descriptor.Descriptor(
  name='WayPointConstraint',
  full_name='motion_spec.WayPointConstraint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='motion_spec.WayPointConstraint.type', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='factor', full_name='motion_spec.WayPointConstraint.factor', index=1,
      number=2, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=534,
  serialized_end=621,
)


_CARTPOSE6 = _descriptor.Descriptor(
  name='CartPose6',
  full_name='motion_spec.CartPose6',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='coordinates', full_name='motion_spec.CartPose6.coordinates', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='id', full_name='motion_spec.CartPose6.id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='id_parent', full_name='motion_spec.CartPose6.id_parent', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=623,
  serialized_end=686,
)


_TWIST = _descriptor.Descriptor(
  name='Twist',
  full_name='motion_spec.Twist',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='coordinates', full_name='motion_spec.Twist.coordinates', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=688,
  serialized_end=716,
)


_JOINTPOSITIONS = _descriptor.Descriptor(
  name='JointPositions',
  full_name='motion_spec.JointPositions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='coordinates', full_name='motion_spec.JointPositions.coordinates', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='id', full_name='motion_spec.JointPositions.id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='id_parent', full_name='motion_spec.JointPositions.id_parent', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=718,
  serialized_end=786,
)


_JOINTVELOCITIES = _descriptor.Descriptor(
  name='JointVelocities',
  full_name='motion_spec.JointVelocities',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='coordinates', full_name='motion_spec.JointVelocities.coordinates', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=788,
  serialized_end=826,
)


_TOOLTIPOFFSET = _descriptor.Descriptor(
  name='ToolTipOffset',
  full_name='motion_spec.ToolTipOffset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='offsetX', full_name='motion_spec.ToolTipOffset.offsetX', index=0,
      number=1, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='offsetY', full_name='motion_spec.ToolTipOffset.offsetY', index=1,
      number=2, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='offsetZ', full_name='motion_spec.ToolTipOffset.offsetZ', index=2,
      number=3, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='offsetA', full_name='motion_spec.ToolTipOffset.offsetA', index=3,
      number=4, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='offsetB', full_name='motion_spec.ToolTipOffset.offsetB', index=4,
      number=5, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='offsetC', full_name='motion_spec.ToolTipOffset.offsetC', index=5,
      number=6, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=828,
  serialized_end=945,
)


_CARTWAYPOINT = _descriptor.Descriptor(
  name='CartWaypoint',
  full_name='motion_spec.CartWaypoint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pose', full_name='motion_spec.CartWaypoint.pose', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='constraint', full_name='motion_spec.CartWaypoint.constraint', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='segmentVelocity', full_name='motion_spec.CartWaypoint.segmentVelocity', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=948,
  serialized_end=1078,
)


_JOINTWAYPOINT = _descriptor.Descriptor(
  name='JointWaypoint',
  full_name='motion_spec.JointWaypoint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pose', full_name='motion_spec.JointWaypoint.pose', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='constraint', full_name='motion_spec.JointWaypoint.constraint', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='segmentVelocity', full_name='motion_spec.JointWaypoint.segmentVelocity', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1081,
  serialized_end=1217,
)


_MOVEC = _descriptor.Descriptor(
  name='MoveC',
  full_name='motion_spec.MoveC',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='constraint', full_name='motion_spec.MoveC.constraint', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='referenceJoint', full_name='motion_spec.MoveC.referenceJoint', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='waypoints', full_name='motion_spec.MoveC.waypoints', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='angle', full_name='motion_spec.MoveC.angle', index=3,
      number=4, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='userframe', full_name='motion_spec.MoveC.userframe', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1220,
  serialized_end=1439,
)


_MOVEL = _descriptor.Descriptor(
  name='MoveL',
  full_name='motion_spec.MoveL',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='constraint', full_name='motion_spec.MoveL.constraint', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='referenceJoint', full_name='motion_spec.MoveL.referenceJoint', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='waypoints', full_name='motion_spec.MoveL.waypoints', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='userframe', full_name='motion_spec.MoveL.userframe', index=3,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1442,
  serialized_end=1646,
)


_MOVEJ = _descriptor.Descriptor(
  name='MoveJ',
  full_name='motion_spec.MoveJ',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='constraint', full_name='motion_spec.MoveJ.constraint', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='waypoints', full_name='motion_spec.MoveJ.waypoints', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1648,
  serialized_end=1753,
)


_SET = _descriptor.Descriptor(
  name='Set',
  full_name='motion_spec.Set',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='motion_spec.Set.path', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='motion_spec.Set.value', index=1,
      number=2, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='time_sec', full_name='motion_spec.Set.time_sec', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1755,
  serialized_end=1807,
)


_GET = _descriptor.Descriptor(
  name='Get',
  full_name='motion_spec.Get',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='motion_spec.Get.path', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1809,
  serialized_end=1828,
)


_WAIT = _descriptor.Descriptor(
  name='Wait',
  full_name='motion_spec.Wait',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='motion_spec.Wait.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='motion_spec.Wait.value', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='timeout', full_name='motion_spec.Wait.timeout', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='error_code', full_name='motion_spec.Wait.error_code', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1830,
  serialized_end=1902,
)


_MOTIONCOMMAND = _descriptor.Descriptor(
  name='MotionCommand',
  full_name='motion_spec.MotionCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='motion_spec.MotionCommand.id', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='commandtype', full_name='motion_spec.MotionCommand.commandtype', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='commandarguments', full_name='motion_spec.MotionCommand.commandarguments', index=2,
      number=3, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1904,
  serialized_end=2003,
)


_MOTIONPROGRAM = _descriptor.Descriptor(
  name='MotionProgram',
  full_name='motion_spec.MotionProgram',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='motion_spec.MotionProgram.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='id', full_name='motion_spec.MotionProgram.id', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='commandlist', full_name='motion_spec.MotionProgram.commandlist', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2005,
  serialized_end=2095,
)


_CARTTOJOINT = _descriptor.Descriptor(
  name='CartToJoint',
  full_name='motion_spec.CartToJoint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='cartpose', full_name='motion_spec.CartToJoint.cartpose', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='jointpose', full_name='motion_spec.CartToJoint.jointpose', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='frame_type', full_name='motion_spec.CartToJoint.frame_type', index=2,
      number=3, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='jointtwist', full_name='motion_spec.CartToJoint.jointtwist', index=3,
      number=4, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='carttwist', full_name='motion_spec.CartToJoint.carttwist', index=4,
      number=5, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2098,
  serialized_end=2341,
)


_CARTTOJOINTLIST = _descriptor.Descriptor(
  name='CartToJointList',
  full_name='motion_spec.CartToJointList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='carttojointlist', full_name='motion_spec.CartToJointList.carttojointlist', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2343,
  serialized_end=2411,
)


_JOINTTOCART = _descriptor.Descriptor(
  name='JointToCart',
  full_name='motion_spec.JointToCart',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='cartpose', full_name='motion_spec.JointToCart.cartpose', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='jointpose', full_name='motion_spec.JointToCart.jointpose', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='jointtwist', full_name='motion_spec.JointToCart.jointtwist', index=2,
      number=3, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='carttwist', full_name='motion_spec.JointToCart.carttwist', index=3,
      number=4, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2414,
  serialized_end=2606,
)


_JOINTTOCARTLIST = _descriptor.Descriptor(
  name='JointToCartList',
  full_name='motion_spec.JointToCartList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='jointtocartlist', full_name='motion_spec.JointToCartList.jointtocartlist', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2608,
  serialized_end=2676,
)

_MOTIONCONSTRAINT.fields_by_name['type'].enum_type = _MOTIONCONSTRAINTTYPE
_MOTIONCONSTRAINT.fields_by_name['time_values'].message_type = _TIMEANDTIMEACCCONSTRAINTVALUES
_MOTIONCONSTRAINT.fields_by_name['timeacc_values'].message_type = _TIMEANDACCCONSTRAINTVALUES
_MOTIONCONSTRAINT.fields_by_name['velacc_values'].message_type = _VELANDACCCONSTRAINTVALUES
_WAYPOINTCONSTRAINT.fields_by_name['type'].enum_type = _WAYPOINTCONSTRAINTTYPE
_CARTWAYPOINT.fields_by_name['pose'].message_type = _CARTPOSE6
_CARTWAYPOINT.fields_by_name['constraint'].message_type = _WAYPOINTCONSTRAINT
_JOINTWAYPOINT.fields_by_name['pose'].message_type = _JOINTPOSITIONS
_JOINTWAYPOINT.fields_by_name['constraint'].message_type = _WAYPOINTCONSTRAINT
_MOVEC.fields_by_name['constraint'].message_type = _MOTIONCONSTRAINT
_MOVEC.fields_by_name['referenceJoint'].message_type = _JOINTPOSITIONS
_MOVEC.fields_by_name['waypoints'].message_type = _CARTWAYPOINT
_MOVEC.fields_by_name['userframe'].enum_type = _USERFRAMETYPE
_MOVEL.fields_by_name['constraint'].message_type = _MOTIONCONSTRAINT
_MOVEL.fields_by_name['referenceJoint'].message_type = _JOINTPOSITIONS
_MOVEL.fields_by_name['waypoints'].message_type = _CARTWAYPOINT
_MOVEL.fields_by_name['userframe'].enum_type = _USERFRAMETYPE
_MOVEJ.fields_by_name['constraint'].message_type = _MOTIONCONSTRAINT
_MOVEJ.fields_by_name['waypoints'].message_type = _JOINTWAYPOINT
_MOTIONCOMMAND.fields_by_name['commandtype'].enum_type = _MOTIONTYPE
_MOTIONPROGRAM.fields_by_name['commandlist'].message_type = _MOTIONCOMMAND
_CARTTOJOINT.fields_by_name['cartpose'].message_type = _CARTPOSE6
_CARTTOJOINT.fields_by_name['jointpose'].message_type = _JOINTPOSITIONS
_CARTTOJOINT.fields_by_name['frame_type'].enum_type = _FEATUREFRAMETYPE
_CARTTOJOINT.fields_by_name['jointtwist'].message_type = _JOINTVELOCITIES
_CARTTOJOINT.fields_by_name['carttwist'].message_type = _TWIST
_CARTTOJOINTLIST.fields_by_name['carttojointlist'].message_type = _CARTTOJOINT
_JOINTTOCART.fields_by_name['cartpose'].message_type = _CARTPOSE6
_JOINTTOCART.fields_by_name['jointpose'].message_type = _JOINTPOSITIONS
_JOINTTOCART.fields_by_name['jointtwist'].message_type = _JOINTVELOCITIES
_JOINTTOCART.fields_by_name['carttwist'].message_type = _TWIST
_JOINTTOCARTLIST.fields_by_name['jointtocartlist'].message_type = _JOINTTOCART
DESCRIPTOR.message_types_by_name['VelAndAccConstraintValues'] = _VELANDACCCONSTRAINTVALUES
DESCRIPTOR.message_types_by_name['TimeAndAccConstraintValues'] = _TIMEANDACCCONSTRAINTVALUES
DESCRIPTOR.message_types_by_name['TimeAndTimeAccConstraintValues'] = _TIMEANDTIMEACCCONSTRAINTVALUES
DESCRIPTOR.message_types_by_name['MotionConstraint'] = _MOTIONCONSTRAINT
DESCRIPTOR.message_types_by_name['WayPointConstraint'] = _WAYPOINTCONSTRAINT
DESCRIPTOR.message_types_by_name['CartPose6'] = _CARTPOSE6
DESCRIPTOR.message_types_by_name['Twist'] = _TWIST
DESCRIPTOR.message_types_by_name['JointPositions'] = _JOINTPOSITIONS
DESCRIPTOR.message_types_by_name['JointVelocities'] = _JOINTVELOCITIES
DESCRIPTOR.message_types_by_name['ToolTipOffset'] = _TOOLTIPOFFSET
DESCRIPTOR.message_types_by_name['CartWaypoint'] = _CARTWAYPOINT
DESCRIPTOR.message_types_by_name['JointWaypoint'] = _JOINTWAYPOINT
DESCRIPTOR.message_types_by_name['MoveC'] = _MOVEC
DESCRIPTOR.message_types_by_name['MoveL'] = _MOVEL
DESCRIPTOR.message_types_by_name['MoveJ'] = _MOVEJ
DESCRIPTOR.message_types_by_name['Set'] = _SET
DESCRIPTOR.message_types_by_name['Get'] = _GET
DESCRIPTOR.message_types_by_name['Wait'] = _WAIT
DESCRIPTOR.message_types_by_name['MotionCommand'] = _MOTIONCOMMAND
DESCRIPTOR.message_types_by_name['MotionProgram'] = _MOTIONPROGRAM
DESCRIPTOR.message_types_by_name['CartToJoint'] = _CARTTOJOINT
DESCRIPTOR.message_types_by_name['CartToJointList'] = _CARTTOJOINTLIST
DESCRIPTOR.message_types_by_name['JointToCart'] = _JOINTTOCART
DESCRIPTOR.message_types_by_name['JointToCartList'] = _JOINTTOCARTLIST
DESCRIPTOR.enum_types_by_name['MOTIONTYPE'] = _MOTIONTYPE
DESCRIPTOR.enum_types_by_name['USERFRAMETYPE'] = _USERFRAMETYPE
DESCRIPTOR.enum_types_by_name['WAYPOINTCONSTRAINTTYPE'] = _WAYPOINTCONSTRAINTTYPE
DESCRIPTOR.enum_types_by_name['MOTIONCONSTRAINTTYPE'] = _MOTIONCONSTRAINTTYPE
DESCRIPTOR.enum_types_by_name['FEATUREFRAMETYPE'] = _FEATUREFRAMETYPE

VelAndAccConstraintValues = _reflection.GeneratedProtocolMessageType('VelAndAccConstraintValues', (_message.Message,), dict(
  DESCRIPTOR = _VELANDACCCONSTRAINTVALUES,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.VelAndAccConstraintValues)
  ))
_sym_db.RegisterMessage(VelAndAccConstraintValues)

TimeAndAccConstraintValues = _reflection.GeneratedProtocolMessageType('TimeAndAccConstraintValues', (_message.Message,), dict(
  DESCRIPTOR = _TIMEANDACCCONSTRAINTVALUES,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.TimeAndAccConstraintValues)
  ))
_sym_db.RegisterMessage(TimeAndAccConstraintValues)

TimeAndTimeAccConstraintValues = _reflection.GeneratedProtocolMessageType('TimeAndTimeAccConstraintValues', (_message.Message,), dict(
  DESCRIPTOR = _TIMEANDTIMEACCCONSTRAINTVALUES,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.TimeAndTimeAccConstraintValues)
  ))
_sym_db.RegisterMessage(TimeAndTimeAccConstraintValues)

MotionConstraint = _reflection.GeneratedProtocolMessageType('MotionConstraint', (_message.Message,), dict(
  DESCRIPTOR = _MOTIONCONSTRAINT,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.MotionConstraint)
  ))
_sym_db.RegisterMessage(MotionConstraint)

WayPointConstraint = _reflection.GeneratedProtocolMessageType('WayPointConstraint', (_message.Message,), dict(
  DESCRIPTOR = _WAYPOINTCONSTRAINT,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.WayPointConstraint)
  ))
_sym_db.RegisterMessage(WayPointConstraint)

CartPose6 = _reflection.GeneratedProtocolMessageType('CartPose6', (_message.Message,), dict(
  DESCRIPTOR = _CARTPOSE6,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.CartPose6)
  ))
_sym_db.RegisterMessage(CartPose6)

Twist = _reflection.GeneratedProtocolMessageType('Twist', (_message.Message,), dict(
  DESCRIPTOR = _TWIST,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.Twist)
  ))
_sym_db.RegisterMessage(Twist)

JointPositions = _reflection.GeneratedProtocolMessageType('JointPositions', (_message.Message,), dict(
  DESCRIPTOR = _JOINTPOSITIONS,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.JointPositions)
  ))
_sym_db.RegisterMessage(JointPositions)

JointVelocities = _reflection.GeneratedProtocolMessageType('JointVelocities', (_message.Message,), dict(
  DESCRIPTOR = _JOINTVELOCITIES,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.JointVelocities)
  ))
_sym_db.RegisterMessage(JointVelocities)

ToolTipOffset = _reflection.GeneratedProtocolMessageType('ToolTipOffset', (_message.Message,), dict(
  DESCRIPTOR = _TOOLTIPOFFSET,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.ToolTipOffset)
  ))
_sym_db.RegisterMessage(ToolTipOffset)

CartWaypoint = _reflection.GeneratedProtocolMessageType('CartWaypoint', (_message.Message,), dict(
  DESCRIPTOR = _CARTWAYPOINT,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.CartWaypoint)
  ))
_sym_db.RegisterMessage(CartWaypoint)

JointWaypoint = _reflection.GeneratedProtocolMessageType('JointWaypoint', (_message.Message,), dict(
  DESCRIPTOR = _JOINTWAYPOINT,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.JointWaypoint)
  ))
_sym_db.RegisterMessage(JointWaypoint)

MoveC = _reflection.GeneratedProtocolMessageType('MoveC', (_message.Message,), dict(
  DESCRIPTOR = _MOVEC,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.MoveC)
  ))
_sym_db.RegisterMessage(MoveC)

MoveL = _reflection.GeneratedProtocolMessageType('MoveL', (_message.Message,), dict(
  DESCRIPTOR = _MOVEL,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.MoveL)
  ))
_sym_db.RegisterMessage(MoveL)

MoveJ = _reflection.GeneratedProtocolMessageType('MoveJ', (_message.Message,), dict(
  DESCRIPTOR = _MOVEJ,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.MoveJ)
  ))
_sym_db.RegisterMessage(MoveJ)

Set = _reflection.GeneratedProtocolMessageType('Set', (_message.Message,), dict(
  DESCRIPTOR = _SET,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.Set)
  ))
_sym_db.RegisterMessage(Set)

Get = _reflection.GeneratedProtocolMessageType('Get', (_message.Message,), dict(
  DESCRIPTOR = _GET,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.Get)
  ))
_sym_db.RegisterMessage(Get)

Wait = _reflection.GeneratedProtocolMessageType('Wait', (_message.Message,), dict(
  DESCRIPTOR = _WAIT,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.Wait)
  ))
_sym_db.RegisterMessage(Wait)

MotionCommand = _reflection.GeneratedProtocolMessageType('MotionCommand', (_message.Message,), dict(
  DESCRIPTOR = _MOTIONCOMMAND,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.MotionCommand)
  ))
_sym_db.RegisterMessage(MotionCommand)

MotionProgram = _reflection.GeneratedProtocolMessageType('MotionProgram', (_message.Message,), dict(
  DESCRIPTOR = _MOTIONPROGRAM,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.MotionProgram)
  ))
_sym_db.RegisterMessage(MotionProgram)

CartToJoint = _reflection.GeneratedProtocolMessageType('CartToJoint', (_message.Message,), dict(
  DESCRIPTOR = _CARTTOJOINT,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.CartToJoint)
  ))
_sym_db.RegisterMessage(CartToJoint)

CartToJointList = _reflection.GeneratedProtocolMessageType('CartToJointList', (_message.Message,), dict(
  DESCRIPTOR = _CARTTOJOINTLIST,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.CartToJointList)
  ))
_sym_db.RegisterMessage(CartToJointList)

JointToCart = _reflection.GeneratedProtocolMessageType('JointToCart', (_message.Message,), dict(
  DESCRIPTOR = _JOINTTOCART,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.JointToCart)
  ))
_sym_db.RegisterMessage(JointToCart)

JointToCartList = _reflection.GeneratedProtocolMessageType('JointToCartList', (_message.Message,), dict(
  DESCRIPTOR = _JOINTTOCARTLIST,
  __module__ = 'motionSL_pb2'
  # @@protoc_insertion_point(class_scope:motion_spec.JointToCartList)
  ))
_sym_db.RegisterMessage(JointToCartList)


# @@protoc_insertion_point(module_scope)



#!/usr/bin/python3

#
#   Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
#   All rights reserved. Copyright (c) 2016 VECTIONEER.
#

class ParameterTree(object):
    """This class represents a parameter tree, obtained from the server.

        Reference to a parameter tree instance is needed for resolving parameters,
        data types and other information to build a correct request message.
    """

    def __init__(self):
        self.__parameter_tree = []
        self.__parameter_map = dict()
        pass

    def load(self, parameter_tree_msg):
        """Loads a parameter tree from ParameterTreeMsg received from the server

            Args:
                parameter_tree_msg(ParameterTreeMsg): parameter tree message from the server

            Examples:
                >>> parameter_tree = motorcortex.ParameterTree()
                >>> parameter_tree_msg = param_tree_reply.get()
                >>> parameter_tree.load(parameter_tree_msg)
        """

        self.__parameter_tree = parameter_tree_msg.params
        for param in self.__parameter_tree:
            self.__parameter_map[param.path] = param

    def getParameterTree(self):
        """
            Returns:
                list(ParameterInfo): a list of parameter descriptions
        """

        return self.__parameter_tree

    def getInfo(self, parameter_path):
        """
            Args:
                parameter_path(str): path of the parameter

            Returns:
                ParameterInfo: parameter description
        """

        return self.__parameter_map.get(parameter_path)

    def getDataType(self, parameter_path):
        """
            Args:
                parameter_path(str): path of the parameter

            Returns:
                DataType: parameter data type

        """

        info = self.getInfo(parameter_path)
        if info:
            return info.data_type

        return None



#!/usr/bin/python3

#
#   Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
#   All rights reserved. Copyright (c) 2016-2020 VECTIONEER.
#


class Reply(object):
    """Reply handle is a JavaScript-like Promise.

        It is resolved when reply is received with successful status and fails otherwise.
    """

    def __init__(self, future):
        self.__future = future

    def get(self, timeout_ms=None):
        """A blocking call to wait for the reply and returns a value.

            Args:
                timeout_ms(integer): timeout for reply in milliseconds

            Returns:
                A protobuf message with a parameter description and value.

            Examples:
                  >>> param_tree_reply = req.getParameterTree()
                  >>> value = param_tree_reply.get()

        """
        return self.__future.result(timeout_ms / 1000.0 if timeout_ms else None)

    def done(self):
        """
            Returns:
                bool: True if the call was successfully cancelled or finished running.
        """
        return self.__future.done()

    def then(self, received_clb, *args, **kwargs):
        """JavaScript-like promise, which is resolved when reply is received.

                Args:
                    received_clb: callback which is resolved when reply is received.

                Returns:
                    self pointer to add 'catch' callback

                Examples:
                    >>> param_tree_reply.then(lambda reply: print("got reply: %s"%reply))
                    >>>                 .catch(lambda g: print("failed"))
        """
        self.__future.add_done_callback(
            lambda msg: received_clb(msg.result(), *args, **kwargs) if msg.result() else None
        )
        return self

    def catch(self, failed_clb, *args, **kwargs):
        """JavaScript-like promise, which is resolved when receive has failed.

            Args:
                failed_clb: callback which is resolved when receive has failed

            Returns:
                self pointer to add 'then' callback

            Examples:
                >>> param_tree_reply.catch(lambda g: print("failed")).then(lambda reply: print("got reply: %s"%reply))
        """
        self.__future.add_done_callback(
            lambda msg: failed_clb(*args, **kwargs) if not msg.result() else None
        )
        return self



#!/usr/bin/python3

#
#   Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
#   All rights reserved. Copyright (c) 2020 VECTIONEER.
#


from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

exec(open('motorcortex/version.py').read())

setup(name='motorcortex-python',
      version=__version__,
      description='Python bindings for Motorcortex Engine',
      long_description=long_description,
      long_description_content_type="text/markdown",
      author='Alexey Zakharov',
      author_email='alexey.zakharov@vectioneer.com',
      url='https://www.motorcortex.io',
      license='MIT',
      packages=['motorcortex'],
      install_requires=['pynng==0.7.*',
                        'protobuf==3.19.*'],
      include_package_data=True,
      )



#!/usr/bin/python3

#
#   Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
#   All rights reserved. Copyright (c) 2016 VECTIONEER.
#

from collections import namedtuple
from struct import unpack_from
from concurrent.futures import Future, ThreadPoolExecutor
import time

timespec = namedtuple('timespec', 'sec, nsec')
"""Timestamp of the parameter 

Args:
    sec(int): seconds
    nsec(int): nanoseconds
"""

Parameter = namedtuple('Parameter', 'timestamp, value')
"""Parameter value with a timestamp

Args:
    timestamp(timespec): Parameter timestamp
    value(any): Parameter value

"""


def compare_timespec(timestamp1, timestamp2):
    return True if (timestamp1.sec == timestamp2.sec) and (timestamp1.nsec == timestamp2.nsec) else False


"""Compare two timestamps

Args:
    timestamp1(timespec): First timestamp to compare
    timestamp2(timespec): Second timestamp to compare

"""


def timespec_to_sec(timestamp):
    return timestamp.sec + timestamp.nsec / 1000000000.0


"""Convert a timestamp to seconds

Args:
    timestamp(timespec): Timestamp to convert
    
Returns:
    Time in seconds
    
"""


def timespec_to_msec(timestamp):
    return timestamp.sec * 1000 + timestamp.nsec / 1000000


"""Convert a timestamp to milliseconds

Args:
    timestamp(timestamp): Timestamp to convert
    
Returns:
    Time in milliseconds

"""


def timespec_to_usec(timestamp):
    return timestamp.sec * 1000000 + timestamp.nsec / 1000


"""Convert a timestamp to microseconds

Args:
    timestamp(timestamp): Timestamp to convert
    
Returns:
    Time in microseconds

"""


def timespec_to_nsec(timestamp):
    return timestamp.sec * 1000000000 + timestamp.nsec


"""Convert a timestamp to nanoseconds

Args:
    timestamp(timestamp): Timestamp to convert
    
Returns:
    Time in nanoseconds

"""


class Subscription(object):
    """Subscription class represents a group of parameters.

        It returns the latest values and a timestamp of the group.

        Subscription class could be used as an observer, which
        notifies on every update or could be used as polling.
    """

    def __init__(self, group_alias, protobuf_types, frq_divider):
        self.__info = None
        self.__group_alias = group_alias
        self.__protobuf_types = protobuf_types
        self.__decoder = []
        self.__clb_list = []
        self.__future = Future()
        self.__values = None
        self.__layout = None
        self.__is_complete = False
        self.__observer_list = []
        self.__pool = None
        self.__frq_divider = frq_divider

    def id(self):
        """
            Returns:
                int: subscription identifier
        """
        return self.__info.id

    def alias(self):
        """
            Returns:
                str: group alias
        """
        return self.__group_alias

    def frqDivider(self):
        """
            Returns:
                str: frequency divider of the group
        """
        return self.__frq_divider

    def read(self):
        """Read the latest values of the parameters in the group.

            Returns:
                list(Parameter): list of parameters
        """
        return self.__values[:] if self.__is_complete else None

    def layout(self):
        """Get a layout of the group.

            Returns:
                list(str): ordered list of the parameters in the group
        """
        return self.__layout[:] if self.__is_complete else None

    def done(self):
        """
            Returns:
                bool: True if the call was successfully cancelled or finished running.

            Examples:
                >>> subscription = sub.subscribe("root/logger/logOut", "log")
                >>> while not subscription.done():
                >>>     time.sleep(0.1)
        """
        return self.__future.done()

    def get(self, timeout_sec=1.0):
        """
            Returns:
                bool: StatusMsg if the call was successfully, None if timeout happened.

            Examples:
                >>> subscription = sub.subscribe("root/logger/logOut", "log")
                >>> done = subscription.get()
        """

        return self.__future.result(timeout_sec)

    def then(self, subscribed_clb):
        """JavaScript-like promise, which is resolved when subscription is completed.

            Args:
                subscribed_clb: callback which is resolved when subscription is completed.

            Returns:
                self pointer to add 'catch' callback

            Examples:
                >>> subscription = sub.subscribe("root/logger/logOut", "log")
                >>> subscription.then(lambda val: print("got: %s"%val)).catch(lambda d: print("failed"))
        """

        self.__future.add_done_callback(
            lambda msg: subscribed_clb(msg.result()) if msg.result() else None
        )
        return self

    def catch(self, failed):
        """JavaScript-like promise, which is resolved when subscription has failed.

            Args:
                failed: callback which is resolved when subscription has failed

            Returns:
                self pointer to add 'then' callback

            Examples:
                >>> subscription = sub.subscribe("root/logger/logOut", "log")
                >>> subscription.catch(lambda d: print("failed")).then(lambda val: print("got: %s"%val))
        """

        self.__future.add_done_callback(
            lambda msg: failed() if not msg.result() else None
        )
        return self

    def notify(self, observer_list):
        """Set an observer, which is notified on every group update.

            Args:
                observer_list: a callback function (or list of callback functions)
                to notify when new values are available

            Examples:
                  >>> def update(parameters):
                  >>>   print(parameters) #list of Parameter tuples

                  >>> data_sub.notify(update)

        """
        self.__observer_list = observer_list if type(observer_list) is list else [observer_list]
        self.__pool = ThreadPoolExecutor(len(self.__observer_list))

    def _complete(self, msg):
        self.__decoder = []
        self.__values = []
        self.__layout = []
        if msg.status == 0:
            self.__info = msg
            for param in msg.params:
                data_type = param.info.data_type
                self.__decoder.append(self.__protobuf_types.getTypeByHash(data_type))
                self.__values.append(Parameter(timespec(0, 0), [0] * param.info.number_of_elements))
                self.__layout.append(param.info.path)

            self.__is_complete = True
            self.__future.set_result(msg)
            return True
        else:
            self._failed()

        return False

    def _updateId(self, new_id):
        self.__info.id = new_id

    def _updateProtocol0(self, sub_msg, length):
        if sub_msg:
            counter = 0
            n_params = len(self.__info.params)
            for param in self.__info.params:
                offset = param.offset
                size = param.size

                # the last element in the group may have a variable size
                if ((counter + 1) == n_params) and ((offset + size) > length):
                    size = length - offset

                if (offset + size) <= length:
                    timestamp = timespec._make(unpack_from('QQ', sub_msg, offset))
                    value = self.__decoder[counter].decode(sub_msg[offset + 16: offset + size],
                                                           (size - 16) / param.info.data_size)
                    self.__values[counter] = Parameter(timestamp, value)

                counter += 1

            if self.__observer_list:
                value = self.__values[:]
                for observer in self.__observer_list:
                    self.__pool.submit(observer, value)

    def _updateProtocol1(self, sub_msg, length):
        if sub_msg:
            timestamp = timespec._make(unpack_from('QQ', sub_msg, 0))
            counter = 0
            n_params = len(self.__info.params)
            for param in self.__info.params:
                offset = param.offset + 16
                size = param.size

                # the last element in the group may have a variable size
                if ((counter + 1) == n_params) and ((offset + size) > length):
                    size = length - offset

                if (offset + size) <= length:
                    value = self.__decoder[counter].decode(sub_msg[offset: offset + size],
                                                           size / param.info.data_size)
                    self.__values[counter] = Parameter(timestamp, value)

                counter += 1

            if self.__observer_list:
                value = self.__values[:]
                for observer in self.__observer_list:
                    self.__pool.submit(observer, value)

    def _failed(self):
        self.__future.set_result(None)



#!/usr/bin/python3

#
#   Developer : Alexey Zakharov (alexey.zakharov@vectioneer.com)
#   All rights reserved. Copyright (c) 2016 VECTIONEER.
#

from __future__ import unicode_literals
import motorcortex.motorcortex_pb2
import logging
import sys
import os

if sys.version_info[0] >= 3:
    from importlib.machinery import SourceFileLoader

    if sys.version_info[1] < 4:
        def importLibrary(name, path):
            return SourceFileLoader(name, path).load_module()
    else:
        from types import ModuleType


        def importLibrary(name, path):
            loader = SourceFileLoader(name, path)
            mod = ModuleType(loader.name)
            return loader.exec_module(mod)
else:
    from builtins import bytes
    from imp import load_source


    def importLibrary(name, path):
        return load_source(name, path)

from json import load
from inspect import ismodule

from collections import namedtuple
import struct

# defines
motorcortex_parameter_msg = 1
motorcortex_parameter_list_msg = 2

ParameterMsg = namedtuple('ParameterMsg', 'header, info, value, status')
"""Data type which represents parameter description and value"""

ParameterListMsg = namedtuple('ParameterListMsg', 'header, params')
"""Data type which represents a list of parameters with descriptions and values"""

StatusCode = dict()
UserLevel = dict()


class PrimitiveTypes(object):
    def __init__(self, type_name):
        self.__decoder = None
        self.__encoder = None
        self.__prepare = None
        if (type_name == "BOOL"):
            self.__decoder = lambda len, val: struct.unpack('%db' % len, val)
            self.__encoder = lambda len, val: struct.pack('%db' % len, *val)
            self.__prepare = lambda val: [bool(x) for x in val.split(",")]
        elif (type_name == "DOUBLE"):
            self.__decoder = lambda len, val: struct.unpack('%dd' % len, val)
            self.__encoder = lambda len, val: struct.pack('%dd' % len, *val)
            self.__prepare = lambda val: [float(x) for x in val.split(",")]
        elif (type_name == "FLOAT"):
            self.__decoder = lambda len, val: struct.unpack('%df' % len, val)
            self.__encoder = lambda len, val: struct.pack('%df' % len, *val)
            self.__prepare = lambda val: [float(x) for x in val.split(",")]
        elif (type_name == "INT8"):
            self.__decoder = lambda len, val: struct.unpack('%db' % len, val)
            self.__encoder = lambda len, val: struct.pack('%db' % len, *val)
            self.__prepare = lambda val: [int(float(x)) & 0x7f for x in val.split(",")]
        elif (type_name == "UINT8"):
            self.__decoder = lambda len, val: struct.unpack('%dB' % len, val)
            self.__encoder = lambda len, val: struct.pack('%dB' % len, *val)
            self.__prepare = lambda val: [int(float(x)) & 0xff for x in val.split(",")]
        elif (type_name == "INT16"):
            self.__decoder = lambda len, val: struct.unpack('%dh' % len, val)
            self.__encoder = lambda len, val: struct.pack('%dh' % len, *val)
            self.__prepare = lambda val: [int(float(x)) & 0x7fff for x in val.split(",")]
        elif (type_name == "UINT16"):
            self.__decoder = lambda len, val: struct.unpack('%dH' % len, val)
            self.__encoder = lambda len, val: struct.pack('%dH' % len, *val)
            self.__prepare = lambda val: [int(float(x)) & 0xffff for x in val.split(",")]
        elif (type_name == "INT32"):
            self.__decoder = lambda len, val: struct.unpack('%di' % len, val)
            self.__encoder = lambda len, val: struct.pack('%di' % len, *val)
            self.__prepare = lambda val: [int(float(x)) & 0x7fffffff for x in val.split(",")]
        elif (type_name == "UINT32"):
            self.__decoder = lambda len, val: struct.unpack('%dI' % len, val)
            self.__encoder = lambda len, val: struct.pack('%dI' % len, *val)
            self.__prepare = lambda val: [int(float(x)) & 0xffffffff for x in val.split(",")]
        elif (type_name == "INT64"):
            self.__decoder = lambda len, val: struct.unpack('%dq' % len, val)
            self.__encoder = lambda len, val: struct.pack('%dq' % len, *val)
            self.__prepare = lambda val: [int(float(x)) for x in val.split(",")]
        elif (type_name == "UINT64"):
            self.__decoder = lambda len, val: struct.unpack('%dQ' % len, val)
            self.__encoder = lambda len, val: struct.pack('%dQ' % len, *val)
            self.__prepare = lambda val: [int(float(x)) for x in val.split(",")]
        elif (type_name == "STRING"):
            self.__decoder = lambda len, val: val.decode()
            self.__encoder = lambda len, val: val.encode()
        elif (type_name == "USER_TYPE"):
            self.__decoder = lambda len, val: val
            self.__encoder = lambda len, val: val

    def decode(self, value, number_of_elements):
        val = self.__decoder(number_of_elements, value)

        return val

    def encode(self, value):
        t = type(value)
        # if bytes do nothing
        if t is bytes:
            return value

        # if string prepare and encode
        if t is str:
            return self.__encoder(-1, value)

        # if not list convert to list
        if t is not list:
            value = [value]

        # encode
        values = self.__encoder(len(value), value)

        return values


class ModuleHashListPairContainer(object):
    def __init__(self, module, hash_list):
        self.module = module
        self.hash_list = hash_list


class MessageTypes(object):
    """Class for handling motorcortex data types, loads proto files and hash files,
    creates a dictionary with all available data types, resolves data types by,
    name or by hash, performs encoding and decoding of the messages.

    """

    def __init__(self):
        self.__types_by_hash = dict()
        self.__hashes_by_name = dict()
        self.__hash_len = 4
        self.__module_hash_pair_list = []
        # Default motorcortex messages are loaded from the library folder.
        # They could be replaced by load command if needed
        path = os.path.dirname(motorcortex.motorcortex_pb2.__file__)
        self.load([{'proto': motorcortex.motorcortex_pb2, 'hash': os.sep.join([path, 'motorcortex_hash.json'])}])

    def motorcortex(self):
        """Returns default motorcortex messages, provided with the package.
        System messages could be replaced at runtime with a newer version,
        by load([{'proto': 'path to the new message proto', 'hash': 'path to the new message hash'}])

        Returns:
            returns motorcortex messages
        """
        return self.getNamespace('motorcortex')

    def load(self, proto_hash_pair_list=None):
        """Loads an array of .proto and .json file pairs.
            Args:
                proto_hash_pair_list([{'hash'-`str`,'proto'-`str`}]): list of hash and proto messages

            Returns:
                list(Module): list of loaded modules with protobuf messages.

            Examples:
                >>> motorcortex_msg, motionsl_msg = motorcortex_types.load(
                >>>     # motorcortex hashes and messages
                >>>     [{'proto': './motorcortex-msg/motorcortex_pb2.py',
                >>>       'hash': './motorcortex-msg/motorcortex_hash.json'},
                >>>     # robot motion hashes and messages
                >>>      {'proto': './motorcortex-msg/motionSL_pb2.py',
                >>>       'hash': './motorcortex-msg/motionSL_hash.json'}])

        """
        count = 0
        module_hash_pair_list = []
        for proto_hash_pair in proto_hash_pair_list:
            with open(proto_hash_pair['hash']) as data_file:
                hash = load(data_file)
                proto = proto_hash_pair['proto']
                if ismodule(proto):
                    logging.debug(f"Adding module: {proto}")
                    module_hash_pair_list.append(ModuleHashListPairContainer(proto, hash))
                elif isinstance(proto, str):
                    logging.debug(f"Loading module: {proto}")

                    name = os.path.splitext(os.path.basename(proto))[0]
                    module_hash_pair_list.append(
                        ModuleHashListPairContainer(importLibrary(str(name), proto), hash))
                else:
                    logging.error(f"Failed to load module: {proto}")
                    return
                count = count + 1

        for module_hash_pair in module_hash_pair_list:
            for hash_type_pair in module_hash_pair.hash_list:
                # getting hash and converting to int
                hash = int(hash_type_pair['hash'], 16)
                # creating dict
                self.__hashes_by_name[hash_type_pair['type']] = hash

                module = module_hash_pair.module
                # searching for messages from hash file
                for name in module.__dict__:
                    type = module.__dict__[name]
                    if hasattr(type, 'DESCRIPTOR'):
                        if hasattr(type.DESCRIPTOR, 'full_name'):
                            msg_name = type.DESCRIPTOR.full_name
                            if (msg_name == hash_type_pair['type']):
                                setattr(type, "decode_value", 0)
                                self.__types_by_hash[hash] = type

            self.__loadPrimitives("DataType", module)
            self.__loadEnum("ParameterFlag", module, motorcortex)
            self.__loadEnum("ParameterType", module, motorcortex)
            self.__loadEnum("Permission", module, motorcortex)
            self.__loadEnum("StatusCode", module, motorcortex)
            self.__loadEnum("UserGroup", module, motorcortex)
            self.__loadEnum("Unit", module, motorcortex)

            param_msg_type = self.getTypeByName("motorcortex.ParameterMsg")
            if param_msg_type:
                param_msg_type.decode_value = motorcortex_parameter_msg

            param_list_msg_type = self.getTypeByName("motorcortex.ParameterListMsg")
            if param_list_msg_type:
                param_list_msg_type.decode_value = motorcortex_parameter_list_msg

            for module_hash in self.__module_hash_pair_list:
                module_name = module_hash.module.DESCRIPTOR.package
                if module_name == module_hash_pair.module.DESCRIPTOR.package:
                    logging.warning(f"Module {module_name} already exists, replacing it with a new version")
                    self.__module_hash_pair_list.remove(module_hash)

        self.__module_hash_pair_list += module_hash_pair_list
        return [el.module for el in module_hash_pair_list]

    def __loadPrimitives(self, name, object):
        try:
            primitive_data_type = getattr(object, name)
            for key in primitive_data_type.DESCRIPTOR.values:
                hash = key.number
                name = key.name
                if hash > 0:
                    self.__hashes_by_name[name] = hash
                    self.__types_by_hash[hash] = PrimitiveTypes(name)
            logging.debug(f"Loaded types from {name}")
        except:
            pass

    def __loadEnum(self, enum_name, object, module):
        try:
            enum = getattr(object, enum_name)
            for key in enum.DESCRIPTOR.values:
                setattr(module, key.name, key.number)
            logging.debug(f"Loaded enumerator {enum_name}")
        except:
            pass

    def createType(self, type_name):
        """Returns an instance of the loaded data type given type name.

            Args:
                type_name(str): type name

            Returns:
                an instance of the requested type.

        """
        type = self.getTypeByName(type_name)
        if type:
            return type()

        return None

    def getTypeByHash(self, id):
        """Returns a data type given its hash.

            Args:
                id(int): type hash

            Returns:
                requested data type.
        """

        return self.__types_by_hash.get(id)

    def getTypeByName(self, name):
        """Returns a data type given its name.

            Args:
                name(str): type name

            Returns:
                requested data type.
        """

        return self.__types_by_hash.get(self.__hashes_by_name.get(name))

    def getNamespace(self, name):
        """Returns a module/namespace with data types.

            Args:
                name(str): module name

            Returns:
                requested module

            Examples:
                >>> # loading module motion_spec
                >>> MotionSpecType = motorcortex_types.getNamespace("motion_spec")
                >>> # instantiating a motion program from the module
                >>> motion_program = MotionSpecType.MotionProgram()
        """

        for el in self.__module_hash_pair_list:
            if el.module.DESCRIPTOR.package == name:
                return el.module

        return None

    def decode(self, wire_data):
        """Decodes data received from the server"""

        hash = wire_data[0] + (wire_data[1] << 8) + (wire_data[2] << 16) + (wire_data[3] << 24)
        type_inst = self.__types_by_hash[hash]
        buf = wire_data[self.__hash_len:]
        msg = type_inst()
        msg.ParseFromString(buf)

        if type_inst.decode_value == motorcortex_parameter_msg:
            if msg.status == motorcortex.OK:
                value_type = self.__types_by_hash[msg.info.data_type]
                return ParameterMsg(msg.header, msg.info, value_type.decode(msg.value, msg.info.number_of_elements),
                                    motorcortex.OK)
            else:
                return ParameterMsg(msg.header, msg.info, None, msg.status)
        elif type_inst.decode_value == motorcortex_parameter_list_msg:
            params = []
            for param in msg.params:
                if msg.status == motorcortex.OK:
                    value_type = self.__types_by_hash[param.info.data_type]
                    params.append(ParameterMsg(param.header, param.info,
                                               value_type.decode(param.value, param.info.number_of_elements),
                                               motorcortex.OK))
                else:
                    params.append(ParameterMsg(msg.header, msg.info, None, msg.status))
            return ParameterListMsg(msg.header, params)

        return msg

    def encode(self, proto_data):
        """Encodes data to send to the server"""
        type_name = proto_data.DESCRIPTOR.full_name
        hash = self.__hashes_by_name[type_name]
        wire_type = bytes([hash & 0xff, (hash >> 8) & 0xff, (hash >> 16) & 0xff, (hash >> 24) & 0xff])
        wire_type += proto_data.SerializeToString()

        return wire_type
